import os
import os.path
import time
from boto.s3.connection import S3Connection
from boto.s3.key import Key

### S3 UPLOAD


def upload_directory(dirname, bucketname):
    upload_directory_to_bucket_by_name(dirname, "", bucketname)


def upload_directory_to_bucket_by_name(base, dirname, bucketname, bucketprefix=''):
    "Upload an entire directory to S3."
    
    aws_access_key = os.environ['AWS_ACCESS_KEY']
    aws_secret_key = os.environ['AWS_SECRET_KEY']
    
    s3     = S3Connection(aws_access_key, aws_secret_key)
    bucket = s3.create_bucket(bucketname)
    upload_directory_to_bucket(base, dirname, bucket, bucketprefix)


def upload_directory_to_bucket(base, dirname, bucket, bucketprefix):
    for f in os.listdir(os.path.join(base, dirname)):
        fname = os.path.join(dirname, f)
        if os.path.isdir(os.path.join(base, fname)):
            upload_directory_to_bucket(base, fname, bucket, bucketprefix)
        else:
            upload_file_to_bucket(base, fname, bucket, bucketprefix)


def upload_file_to_bucket_by_name(base, fname, bucketname, keyname=None, bucketprefix='', public=True):
    "Upload a single file to S3."
    
    aws_access_key = os.environ['AWS_ACCESS_KEY']
    aws_secret_key = os.environ['AWS_SECRET_KEY']
    
    s3     = S3Connection(aws_access_key, aws_secret_key)
    bucket = s3.create_bucket(bucketname)
    upload_file_to_bucket(base, fname, bucket, keyname, bucketprefix, public)


def upload_file_to_bucket(base, fname, bucket, keyname=None, bucketprefix='', public=True):
    "Upload a single file to S3."
    
    TTL = 31536000      # one year
    
    print "Uploading %40.40s" % fname,
    while True:
        try:
            k = Key(bucket)
            if keyname is None:
                k.key = os.path.join(bucketprefix, fname)
            else:
                k.key = keyname
            k.set_contents_from_filename(
                os.path.join(base, fname),
                replace=False,
                headers={
                    'Cache-Control': 'max-age=%s' % TTL
                },
            )
            if public:
                k.set_acl('public-read')
            print "done."
            break
        except Exception, e:
            import traceback
            print "FAILED, retrying in 10."
            traceback.print_exc()
            time.sleep(10)



# Use setuptools if we can
try:
    from setuptools.core import setup
except ImportError:
    from distutils.core import setup

PACKAGE = 'awslib'
VERSION = '0.2'

setup(
    name=PACKAGE, version=VERSION,
    description="Small library providing Amazon Web Services utility functions on top of boto",
    packages=[ 'awslib' ],
    license='MIT',
    author='Art Discovery Ltd',
    maintainer='James Aylett',
    maintainer_email='james@tartarus.org',
    install_requires=[
        'boto',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
    ],
)



import os
import boto.ec2.connection
import boto.ec2.elb
import boto.ec2

### EC2 HOSTLIST QUERIES

def hosts_from_security_group(
    group_name,
    display=False,
    region=None,
    aws_access_key=None,
    aws_secret_key=None,
    return_private_dns_names=False,
):
    if aws_access_key is None:
        aws_access_key = os.environ['AWS_ACCESS_KEY']
    if aws_secret_key is None:
        aws_secret_key = os.environ['AWS_SECRET_KEY']
    sg_hosts = []
    
    if region:
        ec2c = boto.ec2.connect_to_region(region, aws_access_key_id=env.access_key_id, aws_secret_access_key=env.secret_access_key)
    else:
        ec2c = boto.ec2.connection.EC2Connection(aws_access_key, aws_secret_key)
    sg   = ec2c.get_all_security_groups( groupnames=[group_name] )[0]
    for i in sg.instances():
        if i.state!='running':
            if display:
                print "Skipping", i.private_dns_name, "(not running)"
            continue
        if return_private_dns_names:
            sg_hosts.append(i.private_dns_name)
        else:
            sg_hosts.append(i.public_dns_name)
        if display:
            print i.private_dns_name
    return sg_hosts


def hosts_from_elb(elb_name):
    aws_access_key = os.environ['AWS_ACCESS_KEY']
    aws_secret_key = os.environ['AWS_SECRET_KEY']

    ec2c = boto.ec2.connection.EC2Connection(aws_access_key, aws_secret_key)
    elbc = boto.ec2.elb.ELBConnection(aws_access_key, aws_secret_key)
    bh = elbc.get_all_load_balancers(elb_name)[0]
    rs = ec2c.get_all_instances(instance_ids=map(lambda x:x.id, bh.instances))
    elb_hosts = []
    for r in rs:
        for i in r.instances:
            if i.state!='running':
                continue
            elb_hosts.append(i.public_dns_name)
    return elb_hosts


def hosts_by_instance_id(ids):
    aws_access_key = os.environ['AWS_ACCESS_KEY']
    aws_secret_key = os.environ['AWS_SECRET_KEY']

    ec2c = boto.ec2.connection.EC2Connection(aws_access_key, aws_secret_key)
    reservations = ec2c.get_all_instances(ids.split('|'))
    hosts = []
    for r in reservations:
        for i in r.instances:
            if i.state!='running':
                continue
            hosts.append(i.public_dns_name)
    return hosts
